@@ -332,6 +332,9 @@ _id3_parse_v2_frame(id3info *id3)
   // If the frame is compressed, it will be decompressed here
   Buffer *decompressed = 0;
   
+  // tag_data_safe flag is used if skipping artwork and artwork is not raw image data (needs unsync)
+  id3->tag_data_safe = 1;
+  
   if ( !_check_buf(id3->infile, id3->buf, 10, ID3_BLOCK_SIZE) ) {
     ret = 0;
     goto out;
@@ -580,6 +583,8 @@ _id3_parse_v2_frame(id3info *id3)
           // Reset decoded_size to 0 since we aren't actually decoding.
           // XXX this would break if we have a compressed + unsync APIC frame but not very likely in the real world
           decoded_size = 0;
+          
+          id3->tag_data_safe = 0;
         }
         else {
           // tested with v2.4-unsync.mp3
@@ -926,12 +931,21 @@ _id3_parse_v2_frame_data(id3info *id3, char const *id, uint32_t size, id3_framet
           
           if (array != NULL && value != NULL && SvPOK(value)) {
             // second+ string, add to array
-            av_push(array, value);
+            // Bug 16452, do not add a null string
+            if (sv_len(value) > 0)
+              av_push(array, value);
           }
         }
         
         if (array != NULL) {
-          my_hv_store( id3->tags, id, newRV_noinc( (SV *)array ) );
+          if (av_len(array) == 0) {
+            // Handle the case where we have multiple empty strings leaving an array of 1
+            my_hv_store( id3->tags, id, av_shift(array) );
+            SvREFCNT_dec(array);
+          }
+          else {
+            my_hv_store( id3->tags, id, newRV_noinc( (SV *)array ) );
+          }
         }
         else if (value != NULL && SvPOK(value)) {
           my_hv_store( id3->tags, id, value );
@@ -1134,6 +1148,11 @@ _id3_parse_v2_frame_data(id3info *id3, char const *id, uint32_t size, id3_framet
           // Special handling for APIC tags when in skip_art mode
           if (skip_art) {
             av_push( framedata, newSVuv(size - read) );
+            
+            // Record offset of APIC image data too, unless the data needs to be unsynchronized
+            if (id3->tag_data_safe)
+              av_push( framedata, newSVuv(id3->size - id3->size_remain + read) );
+            
             _id3_skip(id3, size - read);
             read = size;
           }
